% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scholid_types.R
\name{scholid_types}
\alias{scholid_types}
\title{Supported scholid identifier types}
\usage{
scholid_types()
}
\value{
A character vector of supported identifier type strings.
}
\description{
Returns the set of identifier types supported by the scholid package.
}
\examples{
scholid_types()
"orcid" \%in\% scholid_types()
}
